/*****************************************************************************
*
*  basic_wifi_application.c - CC3000 Slim Driver Implementation.
*  Copyright (C) 2011 Texas Instruments Incorporated - http://www.ti.com/
*
*  Redistribution and use in source and binary forms, with or without
*  modification, are permitted provided that the following conditions
*  are met:
*
*    Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*
*    Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
*    documentation and/or other materials provided with the   
*    distribution.
*
*    Neither the name of Texas Instruments Incorporated nor the names of
*    its contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
*  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
*  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
*  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
*  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*****************************************************************************/

#include "wlan.h" 
#include "evnt_handler.h"    
#include "nvmem.h"
#include "socket.h"
#include "cc3000_common.h"
#include "netapp.h"
#include "spi.h"


#include "inc/hw_types.h"
#include "driverlib/timer.h"
#include "driverlib/rom_map.h"
#include "driverlib/systick.h"

#include "dispatcher.h"
#include "spi_version.h"

#include "application_version.h"
#include "host_driver_version.h"

#define PALTFORM_VERSION						 (3)


#define UART_COMMAND_CC3000_SIMPLE_CONFIG_START	 (0x31)

#define UART_COMMAND_CC3000_CONNECT			 	 (0x32)

#define UART_COMMAND_SOCKET_OPEN			 	 (0x33)

#define UART_COMMAND_SEND_DATA				  	 (0x34)

#define UART_COMMAND_RCV_DATA				  	 (0x35)

#define UART_COMMAND_BSD_BIND					 (0x36)

#define UART_COMMAND_SOCKET_CLOSE			 	 (0x37)

#define UART_COMMAND_IP_CONFIG			 	 	 (0x38)

#define UART_COMMAND_CC3000_DISCONNECT		 	 (0x39)

#define UART_COMMAND_CC3000_DEL_POLICY		 	 (0x61)

#define CC3000_RX_BUFFER_SIZE					  (128)

#define DISABLE										(0)

#define ENABLE										(1)

#define SL_VERSION_LENGTH							(9)

//*****************************************************************************
//
// Defines for setting up the system clock.
//
//*****************************************************************************
#define SYSTICK_PER_MS                1000
#define SYSTICK_PER_US                1000000
#define SYSTICK_PER_NS                1000000000
//
// Define the UART IF buffer
//
extern unsigned char g_ucUARTBuffer[];
extern volatile unsigned char uart_have_cmd;
extern volatile unsigned long g_ulRxBuffCount;

//
// 64 bytes stand for the zero-copy 4 channels: SSI for both TX and RX, UART
//


//
// Simple Config Prefix
//
char aucCC3000_prefix[] = {'T', 'T', 'T'};
volatile unsigned long ulSocket, ulSmartConfigFinished, ulCC3000Connected,ulCC3000DHCP;
//
// Indications that UART command has finished etc
//
unsigned char pucUARTCommandDoneString[] = {'\f', '\r', 'D', 'O', 'N', 'E', '\f', '\r'};
unsigned char pucUARTCommandSimpleConfigDoneString[] = {'\f', '\r','S', 'i', 'm', 'p', 'l', 'e', ' ', 'c', 'o', 'n', 'f', 'i', 'g', ' ',  'D', 'O', 'N', 'E', '\f', '\r'};
unsigned char pucUARTExampleAppString[] = {'\f', '\r','E', 'x', 'a', 'm', 'p', 'l', 'e', ' ', 'A', 'p', 'p', ':', 'd', 'r', 'i', 'v', 'e', 'r', ' ', 'v', 'e', 'r', 's', 'i', 'o', 'n', ' ' };
unsigned char pucUARTNoDataString[] = {'\f', '\r', 'N', 'o', ' ', 'd', 'a', 't', 'a', ' ', 'r', 'e', 'c','e', 'i', 'v', 'e', 'd', '\f', '\r'};
unsigned char pucUARTIllegalCommandString[] = {'\f', '\r', 'I', 'l', 'l', 'e', 'g', 'a', 'l', ' ', 'c', 'o','m', 'm', 'a', 'n', 'd', '\f', '\r'};

//
// Reception from the air, buffer - the max data length  + headers
//
unsigned char pucCC3000_Rx_Buffer[CC3000_RX_BUFFER_SIZE + 100];

//
// LMS969B C1 Flash workaround handler - see Errata
//
#define WorkaroundIntHandler    ((void (*)(void))0x881)

//*****************************************************************************
// 
//! This function is FLASH workaround interrupt handler
//! 
//! @param   none
//! 
//! @return  none
//! 
//! @brief   This function is FLASH workaround interrupt handler
// 
//*****************************************************************************
void
FlashIntHandler(void)
{
    //
    // Call the PATCH Code Interrupt handler
    //
    WorkaroundIntHandler();
}


//*****************************************************************************
//
//! atoc
//!
//! \param  none
//!
//! \return none
//!
//! \brief  Convert nibble to hexdecimal from ASCII
//
//*****************************************************************************
unsigned char
atoc(char data)
{
	unsigned char ucRes;

	if ((data >= 0x30) && (data <= 0x39))
	{
		ucRes = data - 0x30;
	}
	else
	{
		if (data == 'a')
		{
			ucRes = 0x0a;;
		}
		else if (data == 'b')
		{
			ucRes = 0x0b;
		}
		else if (data == 'c')
		{
			ucRes = 0x0c;
		}
		else if (data == 'd')
		{
			ucRes = 0x0d;
		}
		else if (data == 'e')
		{
			ucRes = 0x0e;
		}
		else if (data == 'f')
		{
			ucRes = 0x0f;
		}
	}


	return ucRes;
}


//*****************************************************************************
//
//! atoshort
//!
//! \param  none
//!
//! \return none
//!
//! \brief  Convert 2 nibbles in ASCII into a short number
//
//*****************************************************************************

unsigned short
atoshort(char b1, char b2)
{
	unsigned short usRes;

	usRes = (atoc(b1)) * 16 | atoc(b2);

	return usRes;
}

//*****************************************************************************
//
//! ascii_to_char
//!
//! \param  none
//!
//! \return none
//!
//! \brief  Convert 2 bytes in ASCII into one character
//
//*****************************************************************************

unsigned char
ascii_to_char(char b1, char b2)
{
	unsigned char ucRes;

	ucRes = (atoc(b1)) << 4 | (atoc(b2));

	return ucRes;
}


//*****************************************************************************
//
// This function turns the USER LED ON/OFF
// 
//
//*****************************************************************************
static void SmartConfigLedOn(unsigned long trueFalse)
{
	if(trueFalse)
	{
		MAP_GPIOPinWrite(GPIO_PORTF_BASE, GPIO_PIN_3, PIN_HIGH);
	}
	else
	{
		
		MAP_GPIOPinWrite(GPIO_PORTF_BASE, GPIO_PIN_3, PIN_LOW);
	}
}

//*****************************************************************************
//
// The interrupt handler for the SysTick timer.  This handler is called every 1ms
// 
//
//*****************************************************************************
void
SysTickHandler(void)
{
    static unsigned long ulTickCount = 0;
    //
    // Increment the tick counter.
    //
    ulTickCount++;
    //Process after 500 ms
    if(ulTickCount >= 500)
    {
        //
        // Handle any un-solicited event if required - the function shall be triggered 
        // few times in a second
        //
        hci_unsolicited_event_handler();
        ulTickCount = 1;
    }
}

//*****************************************************************************
//
//! GPIOConfigure
//!
//! \param  none
//!
//! \return none
//!
//! \brief  Cofigure the GPIO
//
//*****************************************************************************
void
GPIOConfigure(void)
{
	/* Configure the system peripheral bus that IRQ & EN pin are map to */
	MAP_SysCtlPeripheralEnable( SYSCTL_PERIPH_IRQ_PORT);
	//
	// Disable all the interrupts before configuring the lines
	//
	MAP_GPIOPinIntDisable(SPI_GPIO_IRQ_BASE, 0xFF);
    //
    // Cofigure WLAN_IRQ pin as input
    //
    MAP_GPIOPinTypeGPIOInput(SPI_GPIO_IRQ_BASE, SPI_IRQ_PIN);

    GPIOPadConfigSet(SPI_GPIO_IRQ_BASE, SPI_IRQ_PIN, GPIO_STRENGTH_8MA,
                   GPIO_PIN_TYPE_STD_WPU);     
    //
    // Setup the GPIO interrupt for this pin
    //
    MAP_GPIOIntTypeSet(SPI_GPIO_IRQ_BASE, SPI_IRQ_PIN, GPIO_FALLING_EDGE);
	//
    // Disable WLAN chip
    //
    MAP_GPIOPinTypeGPIOOutput(SPI_GPIO_IRQ_BASE, SPI_EN_PIN);
    GPIOPadConfigSet(SPI_GPIO_IRQ_BASE, SPI_EN_PIN, GPIO_STRENGTH_8MA,
                   GPIO_PIN_TYPE_STD_WPD);     
    //MAP_GPIOPinWrite(SPI_GPIO_IRQ_BASE, SPI_EN_PIN, PIN_HIGH);
    MAP_GPIOPinWrite(SPI_GPIO_IRQ_BASE, SPI_EN_PIN, PIN_LOW);
    /* This delay is to resolve a communication issue with CC3000
     * when this delay is absent. Likely has to do with power surges
     */
    SysCtlDelay(600000);
	//
    // Disable WLAN CS with pull up Resistor
    //
	MAP_SysCtlPeripheralEnable(SYSCTL_PERIPH_SPI_PORT1);	
    MAP_GPIOPinTypeGPIOOutput(SPI_PORT1, SPI_CS_PIN);
    GPIOPadConfigSet(SPI_PORT1, SPI_CS_PIN, GPIO_STRENGTH_8MA,
                   GPIO_PIN_TYPE_STD_WPU);     
    MAP_GPIOPinWrite(SPI_PORT1, SPI_CS_PIN, PIN_HIGH);
	// 
	// Clear interrupt status
	//
	SpiCleanGPIOISR();
    // 
    // Enable interrupt for WLAN_IRQ pin
    //
    MAP_GPIOPinIntEnable(SPI_GPIO_IRQ_BASE, SPI_IRQ_PIN);
	
	MAP_IntEnable(INT_SPI_GPIO);
}


//*****************************************************************************
//
//! This function: init_spi
//!
//!  \param  buffer
//!
//!  \return none
//!
//!  \brief  initializes an SPI interface
//
//*****************************************************************************

int init_spi(void)
{
	//
	// Update the configuration in the SPI driver
	//
	SpiConfigureHwMapping();
	//Configure SPI GPIO Related Pins 
	GPIOConfigure();
	//Configure SPI
    SSIConfigure(1000000, 1, 50000000);
	return(ESUCCESS);
}


//*****************************************************************************
//
//! sendDriverPatch
//!
//! \param  pointer to the length
//!
//! \return none
//!
//! \brief  The function returns a pointer to the driver patch: since there is no patch yet - 
//!				it returns 0
//
//*****************************************************************************
const unsigned char *sendDriverPatch(unsigned long *Length)
{	
	*Length = 0;
	return NULL;
}


//*****************************************************************************
//
//! sendBootLoaderPatch
//!
//! \param  pointer to the length
//!
//! \return none
//!
//! \brief  The function returns a pointer to the boot loader patch: since there is no patch yet -
//!				it returns 0
//
//*****************************************************************************
const unsigned char *sendBootLoaderPatch(unsigned long *Length)
{
	*Length = 0;
	return NULL;
}

//*****************************************************************************
//
//! sendWLFWPatch
//!
//! \param  pointer to the length
//!
//! \return none
//!
//! \brief  The function returns a pointer to the FW patch: since there is no patch yet - it returns 0
//
//*****************************************************************************

const unsigned char *sendWLFWPatch(unsigned long *Length)
{	
	*Length = 0;
	return NULL;
}

//*****************************************************************************
//
//! ReadWlanInterruptPin
//!
//! \param  none
//!
//! \return none
//!
//! \brief  return wlan interrup pin
//
//*****************************************************************************

long ReadWlanInterruptPin(void)
{
  return ROM_GPIOPinRead(SPI_GPIO_IRQ_BASE, SPI_IRQ_PIN);
}

//*****************************************************************************
//
//! Enable waln IrQ pin
//!
//! \param  none
//!
//! \return none
//!
//! \brief  Nonr
//
//*****************************************************************************


void WlanInterruptEnable()
{
   ROM_GPIOPinIntEnable(SPI_GPIO_IRQ_BASE, SPI_IRQ_PIN);
}

//*****************************************************************************
//
//! Disable waln IrQ pin
//!
//! \param  none
//!
//! \return none
//!
//! \brief  Nonr
//
//*****************************************************************************


void WlanInterruptDisable()
{
  ROM_GPIOPinIntDisable(SPI_GPIO_IRQ_BASE, SPI_IRQ_PIN);
}


//*****************************************************************************
//
//! WriteWlanPin
//!
//! \param  new val
//!
//! \return none
//!
//! \brief  This functions enables and disables the CC3000 Radio
//
//*****************************************************************************

void WriteWlanPin( unsigned char val )
{
	if(val)
	{
  		MAP_GPIOPinWrite(SPI_GPIO_IRQ_BASE, SPI_EN_PIN,PIN_HIGH);
	}
	else
	{
  		MAP_GPIOPinWrite(SPI_GPIO_IRQ_BASE, SPI_EN_PIN, PIN_LOW);
	}
	
}


//*****************************************************************************
//
//! CC3000_UsynchCallback
//!
//! \param  Event type
//!
//! \return none
//!
//! \brief  The function handles asynchronous events that come from CC3000 device 
//!		  and operates a LED4 to have an on-board indication
//
//*****************************************************************************

void CC3000_UsynchCallback(long lEventType, char *pcData, unsigned char ucLength)
{
	if (lEventType == HCI_EVNT_WLAN_ASYNC_SIMPLE_CONFIG_DONE)
	{
		ulSmartConfigFinished = 1;
	}

	if (lEventType == HCI_EVNT_WLAN_UNSOL_CONNECT)
	{
		ulCC3000Connected = 1;
		//turn on LED1
		SmartConfigLedOn(TRUE);
	}

	if (lEventType == HCI_EVNT_WLAN_UNSOL_DISCONNECT)
	{		
		ulCC3000Connected = 0;
		//turn off LED1
		SmartConfigLedOn(FALSE);
	}
	
	if (lEventType == HCI_EVNT_WLAN_UNSOL_DHCP)
	{
		ulCC3000DHCP = 1;
	}
}

//*****************************************************************************
//
//! initDriver
//!
//!  \param  None
//!
//!  \return none
//!
//!  \brief  The function initializes a CC3000 device and triggers it to start operation
//
//*****************************************************************************
int
initDriver(void)
{
	//
	// Send a version string
	//
	unsigned char version_string[SL_VERSION_LENGTH];
	//
	// Enable use of PORTD to toggle LED and disable interrupt on this port
	//
	MAP_SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOF);
	MAP_GPIOPinIntDisable(GPIO_PORTF_BASE, 0xFF);
	//
	// Configure LED4
	//
	MAP_GPIOPinTypeGPIOOutput(GPIO_PORTF_BASE, GPIO_PIN_3);
	MAP_GPIOPinWrite(GPIO_PORTF_BASE, GPIO_PIN_3, PIN_LOW);
	MAP_GPIOPinWrite(GPIO_PORTF_BASE, GPIO_PIN_3, PIN_HIGH);
	MAP_GPIOPinWrite(GPIO_PORTF_BASE, GPIO_PIN_3, PIN_LOW);	
    //
    //init all layers
    //
    init_spi(); 
    //
    // Enable processor interrupts
	//
    MAP_IntMasterEnable(); 
	//
	// WLAN On API Implementation
	//
	wlan_init( CC3000_UsynchCallback, (tFWPatches )sendWLFWPatch, (tDriverPatches )sendDriverPatch, (tBootLoaderPatches)sendBootLoaderPatch, ReadWlanInterruptPin, WlanInterruptEnable, WlanInterruptDisable, WriteWlanPin);
	//
	// Trigger a WLAN device
	//
	wlan_start(0);	
	//
	// Mask out all non-required events from CC3000
	//
	wlan_set_event_mask(HCI_EVNT_WLAN_KEEPALIVE|HCI_EVNT_WLAN_UNSOL_INIT|HCI_EVNT_WLAN_ASYNC_PING_REPORT);

	DispatcherUARTConfigure(50000000);
    SysCtlDelay(1000000);	
	
	DispatcherUartSendPacket(pucUARTExampleAppString, sizeof(pucUARTExampleAppString));
	version_string[0]= 0x30 + PALTFORM_VERSION;
	version_string[1]= '.';
	version_string[2]= 0x30 + APPLICATION_VERSION_NUMBER;
	version_string[3]= '.';
	version_string[4] = 0x30 + SPI_VERSION_NUMBER;
	version_string[5]= '.';
	version_string[6]= 0x30 + DRIVER_VERSION_NUMBER;
	version_string[7]=  '\n';
	version_string[8]=  '\r';
	DispatcherUartSendPacket(version_string, SL_VERSION_LENGTH);	

	return(0);
}



//*****************************************************************************
//
//! StartSmartConfig
//!
//!  \param  None
//!
//!  \return none
//!
//!  \brief  The function triggers a smart configuration process on CC3000.
//!			it exists upon completion of the process
//
//*****************************************************************************

void StartSmartConfig(void)
{
	ulSmartConfigFinished = 0;
	ulCC3000Connected = 0;
	ulCC3000DHCP = 0;
	//
	// Reset all the previous configuration
	//
	wlan_ioctl_set_connection_policy(DISABLE, DISABLE, DISABLE);	
	
	//
	// Trigger the Smart Config process
	//
	SmartConfigLedOn(TRUE);
	wlan_first_time_config_set_prefix(aucCC3000_prefix);
	SmartConfigLedOn(FALSE);		
	//
	// Start the First Time config process
	//
	wlan_first_time_config_start();
 	SmartConfigLedOn(TRUE);       
	//
	// Wait for First Time config finished
	// 
	while (ulSmartConfigFinished == 0)
	{
		SmartConfigLedOn(FALSE);
		SysCtlDelay(16500000);
		SmartConfigLedOn(TRUE);		  
        SysCtlDelay(16500000);
	}
	SmartConfigLedOn(TRUE);	
	//
	// Configure to connect automatically to the AP retrieved in the 
	// First Time config process
	//
	wlan_ioctl_set_connection_policy(DISABLE, DISABLE, ENABLE);
	
	//
	// reset the CC3000
	// 
	wlan_stop();

	DispatcherUartSendPacket(pucUARTCommandSimpleConfigDoneString, sizeof(pucUARTCommandSimpleConfigDoneString));
	
	SysCtlDelay(100000);
	wlan_start(0);	
	//
	// Mask out all non-required events
	//
	wlan_set_event_mask(HCI_EVNT_WLAN_KEEPALIVE|HCI_EVNT_WLAN_UNSOL_INIT|HCI_EVNT_WLAN_ASYNC_PING_REPORT);
}

//*****************************************************************************
//
//! DemoHandleUartCommand
//!
//!  \param  buffer
//!
//!  \return none
//!
//!  \brief  The function handles commands arrived from CLI
//
//*****************************************************************************
void
DemoHandleUartCommand(unsigned char *usBuffer)
{
	char *pcSsid, *pcData, *pcSockAddrAscii;
	unsigned long ulSsidLen, ulDataLength;
	volatile signed long iReturnValue;
	sockaddr tSocketAddr;
	socklen_t tRxPacketLength;
	unsigned char pucIP_Addr[4];
	unsigned char pucIP_DefaultGWAddr[4];
	unsigned char pucSubnetMask[4];
	unsigned char pucDNS[4];

    switch(usBuffer[1])
    {
    	//
    	// Start a smart configuration process
    	//
    	case UART_COMMAND_CC3000_SIMPLE_CONFIG_START:
			StartSmartConfig();
			break;
		
    	//
    	// Start a WLAN Connect process
    	//
		case UART_COMMAND_CC3000_CONNECT:
        {
            ulSsidLen = atoc(usBuffer[2]);
			pcSsid = (char *)&usBuffer[3];
			
            wlan_connect(WLAN_SEC_UNSEC, pcSsid, ulSsidLen, 
            	NULL, NULL, 0);
        } 
		break;

		
    	//
    	// Handle open socket command
    	//
		case UART_COMMAND_SOCKET_OPEN:
		//wait for DHCP process to finish
			while ((ulCC3000DHCP == 0) || (ulCC3000Connected == 0))
			{
				hci_unsolicited_event_handler();
	
				SysCtlDelay(1000);
			}
		    ulSocket = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
			break;

		
    	//
    	// Handle close socket command
    	//
		case UART_COMMAND_SOCKET_CLOSE:
			closesocket(ulSocket);
			ulSocket = 0xFFFFFFFF;
			break;

		
    	//
    	// Handle receive data command
    	//
		case UART_COMMAND_RCV_DATA:
			iReturnValue = recvfrom(ulSocket, pucCC3000_Rx_Buffer, CC3000_RX_BUFFER_SIZE, 0, &tSocketAddr, &tRxPacketLength);
			if (iReturnValue <= 0)
			{
				//
				// No data receibed by device
				//
				DispatcherUartSendPacket(pucUARTNoDataString, sizeof(pucUARTNoDataString));

				
			}
			else
			{
				//
				// Send data to UART...
				//
				DispatcherUartSendPacket(pucCC3000_Rx_Buffer, CC3000_RX_BUFFER_SIZE);

			}
			break;

		
    	//
    	// Handle send data command
    	//
		case UART_COMMAND_SEND_DATA:

			// data pointer
			pcData = (char *)&usBuffer[4];

			// data length to send
			ulDataLength = atoshort(usBuffer[2], usBuffer[3]);
			pcSockAddrAscii = (pcData + ulDataLength);

			// the family is always AF_INET
			tSocketAddr.sa_family = atoshort(pcSockAddrAscii[0], pcSockAddrAscii[1]);

			// the destination port
			tSocketAddr.sa_data[0] = ascii_to_char(pcSockAddrAscii[2], pcSockAddrAscii[3]);
			tSocketAddr.sa_data[1] = ascii_to_char(pcSockAddrAscii[4], pcSockAddrAscii[5]);

			// the destination IP address
			tSocketAddr.sa_data[2] = ascii_to_char(pcSockAddrAscii[6], pcSockAddrAscii[7]);
			tSocketAddr.sa_data[3] = ascii_to_char(pcSockAddrAscii[8], pcSockAddrAscii[9]);
			tSocketAddr.sa_data[4] = ascii_to_char(pcSockAddrAscii[10], pcSockAddrAscii[11]);
			tSocketAddr.sa_data[5] = ascii_to_char(pcSockAddrAscii[12], pcSockAddrAscii[13]);
			
			sendto(ulSocket, pcData, ulDataLength, 0, &tSocketAddr, sizeof(sockaddr));
    		break;

		
    	//
    	// Handle bind command
    	//
		case UART_COMMAND_BSD_BIND:
			tSocketAddr.sa_family = AF_INET;

			// the source port
			tSocketAddr.sa_data[0] = ascii_to_char(usBuffer[2], usBuffer[3]);
			tSocketAddr.sa_data[1] = ascii_to_char(usBuffer[4], usBuffer[5]);

			// all 0 IP address
			memset (&tSocketAddr.sa_data[2], 0, 4);
			
			bind(ulSocket, &tSocketAddr, sizeof(sockaddr));
			
			break;

		
    	//
    	// Handle IP configuration command
    	//
		case UART_COMMAND_IP_CONFIG:
			
			//
			// Network mask is assumed to be 255.255.255.0
			//
			pucSubnetMask[0] = 0xFF;
			pucSubnetMask[1] = 0xFF;
			pucSubnetMask[2] = 0xFF;
			pucSubnetMask[3] = 0x0;

			pucIP_Addr[0] = ascii_to_char(usBuffer[2], usBuffer[3]);
			pucIP_Addr[1] = ascii_to_char(usBuffer[4], usBuffer[5]);
			pucIP_Addr[2] = ascii_to_char(usBuffer[6], usBuffer[7]);
			pucIP_Addr[3] = ascii_to_char(usBuffer[8], usBuffer[9]);

			pucIP_DefaultGWAddr[0] = ascii_to_char(usBuffer[10], usBuffer[11]);
			pucIP_DefaultGWAddr[1] = ascii_to_char(usBuffer[12], usBuffer[13]);
			pucIP_DefaultGWAddr[2] = ascii_to_char(usBuffer[14], usBuffer[15]);
			pucIP_DefaultGWAddr[3] = ascii_to_char(usBuffer[16], usBuffer[17]);

			
			//
			// Currently no implementation of DHCP in hte demo
			//
			pucDNS[0] = 0;
			pucDNS[1] = 0;
			pucDNS[2] = 0;
			pucDNS[3] = 0;
			
			netapp_dhcp((unsigned long *)pucIP_Addr, (unsigned long *)pucSubnetMask, (unsigned long *)pucIP_DefaultGWAddr, (unsigned long *)pucDNS);
			
			break;

		
    	//
    	// Handle WLAN disconnect command
    	//
		case UART_COMMAND_CC3000_DISCONNECT:
			wlan_disconnect();
                        SmartConfigLedOn(FALSE);
			break;

		
    	//
    	// Handle erase policy command
    	//
		case UART_COMMAND_CC3000_DEL_POLICY:
			wlan_ioctl_set_connection_policy(DISABLE, DISABLE, DISABLE);
			break;
			
		default:
			DispatcherUartSendPacket(pucUARTIllegalCommandString, sizeof(pucUARTIllegalCommandString));

			
			break;
			
	}
	//
	// Send a responce - the command handling has finished
	//
	DispatcherUartSendPacket(pucUARTCommandDoneString, sizeof(pucUARTCommandDoneString));
    
}


//*****************************************************************************
//
//! main
//!
//!  \param  None
//!
//!  \return none
//!
//!  \brief  The main loop is executed here
//
//*****************************************************************************

void main(void)
{
	ulCC3000DHCP = 0;
	ulCC3000Connected = 0;
	ulSocket = 0;
	ulSmartConfigFinished=0;

    //
    //  Board Initialization start
    //
	//
	//  16 MHz Crystal on Board. SSI Freq - configure M3 Clock to be ~50 MHz
	//
	MAP_SysCtlClockSet(SYSCTL_SYSDIV_4 | SYSCTL_USE_PLL | SYSCTL_OSC_MAIN | SYSCTL_XTAL_16MHZ);
	//
    // Configure SysTick to occur X times per second, to use as a time
    // reference.  Enable SysTick to generate interrupts.
    //
    SysTickPeriodSet(SysCtlClockGet()/SYSTICK_PER_MS);
    SysTickIntEnable();
    SysTickEnable();
    //
    // Initialize the driver functions and Enable processor interrupts.
    //
    initDriver();
    /* Initialize the UART RX Buffer */  
    memset(g_ucUARTBuffer, 0x00, UART_IF_BUFFER);
    uart_have_cmd =0;		
    // 
    // Loop forever waiting for commands from PC...
    // 
    while(1)
    {	
		if (uart_have_cmd && !(UARTBusy(UART0_BASE)) )
		{   
                     while(UARTBusy(UART0_BASE));
            //
            //Process the cmd in RX buffer
            //
		     DemoHandleUartCommand(g_ucUARTBuffer);
                     
	    //Clear cmd and reset buffer pointer 
	              memset(g_ucUARTBuffer, 0x00, UART_IF_BUFFER);
                      uart_have_cmd = 0;
            
		}
    }  
}
